/**
 * \addtogroup Delay
 * @{
 */

/*****************************************************************************/
/**
 * \file Delay.h
 * Delay routines for 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.2
 * \date 24-05-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef DELAY_H
#define DELAY_H

/** 
 * Hardware crystal frequency in Hz.
 * Adjust \b DELAY_XTAL to match target hardware.
 * \hideinitializer
 */
#define DELAY_XTAL         22118400

/** 
 * X2 mode support binary switch.
 * Set \b DELAY_X2_MODE when CPU X2 clock mode is elsewhere engaged.
 * \hideinitializer
 */
#define DELAY_X2_MODE      0

/*
 * Delay routine timing parameters.
 */
#if (DELAY_X2_MODE == 0)
  #define DELAY_CPU_CLOCK  DELAY_XTAL / 12u   /* 8051 clock rate (X1 mode) */
#else
  #define DELAY_CPU_CLOCK  DELAY_XTAL / 6u    /* 8051 clock rate (X2 mode) */
#endif

#define DELAY_CONST        9.114584e-5        /* Delay routine constant    */
#define DELAY_MULTPLR      (unsigned int)(DELAY_CONST * DELAY_CPU_CLOCK)


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * Millisecond-precision software delay function.
 * Implements a software timing loop based delay with millisecond-precision.
 * See 'Usage' for notes regarding upper bounds on the delay period that may
 * be generated.
 * \param count The number of milliseconds to delay as an \b unsigned \b int.
 * \return -
 */
void delay_ms(volatile unsigned int count);


#endif

/** @} */
